/**************************************************************************************************************
 *
 * Digital Caliper
 *
 * Autor : Finger, maito:finger@fingers-welt.de
 *
 * Controller : Microchip 18F452, 20 Mhz
 * Einstellungen:  OSC=HS, WDT=OFF, PWT=ON, CP=OFF, BOD=ON, LVP=OFF
 *
 * TAB-Space : 2
 * Editor : MI-Edit
 * Entwicklungsumgebung : MPLAB, PIC-C 18
 *
 *************************************************************************************************************/

#include	<pic18.h>
#include 	<string.h>
#include	<ctype.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdio.h>
#include  "defines.h"                                     // globale Deinitionen
#include  "lcd.h"                                         // LCD-Funktionen
#include  "misc.h"                                        // Diverses
#include  "delay.h"                                       // Verzgerungsroutinen
#include  "serial.h"                                      // UART-Funktionen

/**************************************************************************************************
 *
 * Globale Variablen
 *
 **************************************************************************************************/

//-------------
// Allgemeines
//-------------
extern char outpbuf [Buflen];                             // Algemeiner Textstring (siehe Misc.c)
volatile int Warteticker = 0;														  // Zur Syncronisation mit der Schieblehre
int WartetickerAlt = 0;                                   // Ausgabefunktion
volatile char RohErgebniss [6];														// Rohdaten von der Schieblehre
int i = 0;																								// Zhler
char Rohzaehler = 0;																		  // Zum reinholen der Rohdaten
char Leseflag = FALSE;																		// Flag zur Gltigkeit der Rohdaten
char zaehler;																							// Allgemeiner Zaehler
char Taste;                                               // Letzter Tastenzustand
char Kontrast = 0x10;                                     // Kontrasteinstellung
long LongAbsolut = 0;																			// Zusammengesetzte Position
long LongRelativ = 0;                                     // Dito

char Zeichenkette [51];
char Bitzaehler = 0;

int Nullpunkt = 0;                                        // Absoluter Nullpunkt der Messung
int Messpunkt = 0;                                        // Aktuelle Position
int Durch = 0;                                            // Eingegebener Durchmesser

char Durchmesser = FALSE;                                 // Steuerung der Durchmesseranzeige

/*******************************************************************************************************************
 *
 * Zentrale ISR
 *
 *******************************************************************************************************************/
interrupt void INT_ (void)
 {
   //--------------------
   // Timer 1 berlauf ?
   //--------------------
   if (TMR1IF)
   	{

      Systemticker ();                                  // Systemticker erhhen, siehe MISC.C

      TMR1IF=0;                                         // Clear Interrupt-Flag
      TMR1L=0xB4;                                       // Timer 1 neu vorladen (0xF3B4 = 5ms)
      TMR1H=0xF3;                                       // n = 2^16 - t [s] * 625000

   	} // From Timer 1 berlauf

   return;									                            // Zurck zum Hauptprogramm
 }

/***********************************
 *
 * Zeigt den normalen Bildschirm an
 *
 ***********************************/
void Screen (void)
 {
  lcd_clear ();
  lcd_cursorpos (0,0);lcd_puts (" ___.__     ___.__");
  lcd_cursorpos (1,0);lcd_puts (" l [mm]     d [mm]");

  if (Durchmesser)
   {
    lcd_cursorpos (2,0);lcd_puts ("   Durchmesser");
    lcd_cursorpos (3,0);lcd_puts ("    ___.__ mm");
   } else

   {
    lcd_cursorpos (3,0);
    lcd_puts ("www.fingers-welt.de");
   }

 }

/***************************
 *
 * Zeigt einfache Werte an
 *
 ***************************/
void ShowSingle (void)
 {
  int Vorkomma = 0;
  int Nachkomma = 0;

  lcd_cursorpos (0,0);

  // Vorkommastellen Weg
  if (Messpunkt < 0)
   {
    lcd_putch ('-');
    Messpunkt *= -1;
   } else lcd_putch (' ');

  Vorkomma = (Messpunkt/100);
  sprintf (outpbuf, "%03d", Vorkomma);
  lcd_puts (outpbuf);
  lcd_puts (".");

  // Nachkommastellen Weg
  Nachkomma = Messpunkt % 100;
  sprintf (outpbuf, "%02d", Nachkomma);
  lcd_puts (outpbuf);

  lcd_cursorpos (0,12);

  Messpunkt *= 2;

  // Vorkommastellen Durchmesser
  Vorkomma = (Messpunkt/100);
  sprintf (outpbuf, "%03d", Vorkomma);
  lcd_puts (outpbuf);
  lcd_puts (".");

  // Nachkommastellen Durchmesser
  Nachkomma = Messpunkt % 100;
  sprintf (outpbuf, "%02d", Nachkomma);
  lcd_puts (outpbuf);

  lcd_cursorpos (0,18);
  lcd_puts ("  ");
 }

/*************************************************************************************************
 *
 * Globale Initialisierungen
 *
 *************************************************************************************************/
void Init (void)
 {

  //-----------
  // Port-Init
  //-----------

   PORTA=0;
   PORTB=0;
   PORTC=0;
   PORTD=0;
   PORTE=0;

   TRISA = 0;								              	        // NC
   TRISB = 0b11101111;							              	// Alles Eingang bis auf B4
   TRISC = 0b00000000;      	           	  			  // Alles Ausgang
   TRISD = 0;								                        // Alles Ausgang
   TRISE = 0;								                        // NC

   //-----------------------
   // 42 und der ganze Rest
   //-----------------------

   //----------
   // LCD-Init
   //----------

   // Kontrasteinstellung fr das LCD
   PR2 = 0xff;									                    // konfiguriere Timer2 and CCP1 for
   T2CON = 0x15;									                  // 4.88Khz PWM @ 20MHz  (15)
   CCPR1L = Kontrast;									              // initial duty cycle
   CCP1CON = 0x0f;

   // LCD startklar machen
   lcd_init ();									                    // LC-Display initialisieren
   lcd_clear ();

   //------------
   // Timer-Init
   //------------
   TMR1IE=1;									                      // enable Interrupt of TMR1
   T1CON=49;									                      // Prescaler=1:8 + Timer on (49)

   //---------------
   // Interrupts an
   //---------------
   PEIE=1;          							                  // Peripherie Int=on
   GIE=1;										                        // Define Interupts

   //-----------------
   // UART klarmachen
   //-----------------
   UART_Init ();                                    // UART initialisieren

   //--------------------------
   // AD-Wandler-Einstellungen
   //--------------------------
   ADCON0 = 0x00;
   ADCON1 = 0x07;

   //----------------------------
   // Tastenflipflops rcksetzen
   //----------------------------
   PORTB &= ~16;                                    // B4 auf Low
   Delayms (10);                                    // Kurze Pause
   PORTB |= 16;                                     // B4 auf High

   //-----------------------------------
   // Startmeldung via UART und Display
   //-----------------------------------
   put_string_usart ("\n\rDigital Caliper 1.0\n\r");
   lcd_clear ();
   lcd_cursorpos (1,0);
   lcd_puts ("Digital Caliper 1.0");
   lcd_cursorpos (2,0);
   lcd_puts ("www.fingers-welt.de");

   Delayms (2000);

   Screen ();

 }

/*************************************
 *
 * Gibt den Zustand der Tasten wieder
 *
 * 0 : Keine Taste
 * 1 : Taste 0
 * 2 : Taste 1
 * 3 : Beite Tasten
 *
 *************************************/
char GetTaste (void)
 {
  char Puffer = 0;

  if (PORTB & 4) Puffer = 1; else
  if (PORTB & 8) Puffer = 2;  else
  if (PORTB & 12) Puffer = 3;

  if (Puffer != 0)
   {
    Delayms (10);

    sprintf (outpbuf, "Taste %d\n\r", Puffer);
    put_string_usart (outpbuf);

    //----------------------------
    // Tastenflipflops rcksetzen
    //----------------------------
    while ( (PORTB & 4) || (PORTB & 8) )
     {
      PORTB &= ~16;                                    // B4 auf Low
      Delayms (10);                                    // Kurze Pause
      PORTB |= 16;                                     // B4 auf High
     }

    }

  return (Puffer);

 }

/***************************************************************
 *
 * Kaut aus dem Quark von der Schieblehre eine vernnftige Zahl
 *
 ***************************************************************/
void GetMillimeter (void)
 {
  char i;

  long buffer = 0;

  // Negative Zahl ?
  if ( (Zeichenkette [47] + 47) == '0')
   {
     for (i = 24; i < 47; i++) if ( (Zeichenkette [i] + 47) == '1') buffer += ((long)1 << ((long)i-24));
     buffer *= 127;
     buffer /= 1024;
     LongAbsolut = (long)buffer;
   } else

   { // Zahl war negativ
     for (i = 24; i < 47; i++) if ( (Zeichenkette [i] + 47) == '0') buffer += ((long)1 << ((long)i-24));
     buffer *= 127;
     buffer /= 1024;
     LongAbsolut = (long)buffer;
     LongAbsolut *= -1;
   }

   Messpunkt = -1*(int)LongAbsolut + Nullpunkt;

 }

/********************************************
 *
 * Holt einen Wert von der Schieblehre rein
 * FALSE = Fehler
 * Achtung : Absolut zeitkitische Funktion
 *
 ********************************************/
void Empfang (void)
 {
  long Systicker;
  long Startticker;
  char Bitzaehler = 0;

  memset (Zeichenkette,'A',50);

   // Mindestens 40ms mu Clock high sein zur Syncronisation
   Systicker = Get_Ticker ();
   Startticker = Get_Ticker ();
   while ( (Systicker+8) > Get_Ticker () )
    {
     if ( ! (PORTB & 1)) Systicker = Get_Ticker ();

     // Hat sich nach 500ms nichts getan ? Dann raus hier.......
     if ((Startticker + 25) < Get_Ticker ())
      {
       put_string_usart ("X");
       return;
      }
    }

   //-------------------------------------------------
   // Hier angelangt war Clock mindestens 100 ms High
   //-------------------------------------------------
   GIE = 0;                         // Timer abschalten
   while ( (PORTB & 1) );           // Warte, solange 1
   PORTC |= 2;                      // RC1 = 1
   while ( !(PORTB & 1) );          // Warte, solange 0
   PORTC &= ~2;                     // RC1 = 0

   // Jetzt gehts endgltig los
   while (Bitzaehler < 48)
    {

     // Zeichenkette zusammensetzen
     Zeichenkette [Bitzaehler] = (RB6+1);

     // Auf nchste Low-High-Flanke warten
     PORTC |= 1;                    // RC0 = 1
     while ( (PORTB & 1) );         // Warte, solange 1
     PORTC &= ~1;                   // RC0 = 0;
     Bitzaehler++;
     while ( !(PORTB & 1) );        // Warte, solange 0


    } // Von while (Bitzaehler < 47)

     // Letzte Flanke abwarten
     while ( (PORTB & 1) );         // Warte, solange 1
     while ( !(PORTB & 1) );        // Warte, solange 0

   GIE = 1;                         // Timer einschalten

  GetMillimeter ();                 // Umrechnen in Hundersttel

 }

/*************************
 *
 * Nullpunkt definieren
 *
 *************************/
void GetNull (void)
 {
  Empfang ();                                          // Daten reinholen (Das erste mal ist Quark nach dem Einschalten)
  Empfang ();                                          // Daten reinholen
  Nullpunkt = (int)LongAbsolut;                        // Nullpunkt festhalten
 }

/*******************************************
 *
 * Zeigt den aktuellen Werkstckdurchmesser
 * gro und Fett an
 *
 *******************************************/
void ShowDurchmesser (void)
 {
  int Vorkomma;
  int Nachkomma;

  int Messwert;

  lcd_cursorpos (3, 4);
  Messwert = Messpunkt;

  Vorkomma = Messwert/100;
  sprintf (outpbuf, "%03d", Vorkomma);
  lcd_puts (outpbuf);
  lcd_puts (".");

  Nachkomma = Messwert % 100;
  sprintf (outpbuf, "%02d", Nachkomma);
  lcd_puts (outpbuf);
  lcd_puts (" mm");

 }

/**************************************
 *
 * Eingabe eines Werkstckdurchmessers
 *
 **************************************/
void GetDurchmesser (void)
 {
  char Position = 0;
  char Stellen [6] = { 0, 0, 0, 0, 0, 0 };
  char Taste;

  Durchmesser = TRUE;

  // Bildschirm aufbauen
  lcd_clear ();
  lcd_cursorpos (1,0);lcd_puts ("   Durchmesser");
  lcd_cursorpos (2,0);lcd_puts ("    000.00 mm");
  lcd_cursorpos (0,19);lcd_putch (0x7E);
  lcd_cursorpos (3,19);lcd_putch ('+');

  // Cursor einschalten und positionieren
  lcd_cursor_on (TRUE);
  lcd_cursorpos (2, 4);

  // Alle Durchtackern
  while (Position < 6)
   {

    Taste = GetTaste ();

    // Aktuelle Stelle beackern
    if (Taste == 1)
     {
      sprintf (outpbuf, "Pos : %d\n\r ", Position);
      put_string_usart (outpbuf);

      Stellen [Position]++;

      if (Position > 0)
       if (Stellen [Position] > 9) Stellen [Position] = 0;

      if (Position == 0)
       if (Stellen [Position] > 1) Stellen [Position] = 0;

      lcd_cursorpos (2, 4+Position);
      lcd_putch (Stellen [Position]+48);
      lcd_cursorpos (2, 4+Position);
     }

    // Nchste Stelle eingeben
    if (Taste == 2)
     {
      Position++;
      if (Position == 3) Position++;
      lcd_cursorpos (2, 4+Position);
      // position ausgeben
      sprintf (outpbuf, "Pos : %d\n\r ", Position);
      put_string_usart (outpbuf);
     }

   }

  // Cursor aus
  lcd_cursor_on (FALSE);

  // Neuen Durchmesser zusammenbauen
  Durch  = Stellen [0] * 10000;
  Durch += Stellen [1] * 1000;
  Durch += Stellen [2] * 100;
  Durch += Stellen [4] * 10;
  Durch += Stellen [5];

  // Durchmesser testweise ausgeben
  sprintf (outpbuf, "\n\rDurchmesser : %d\n\r", Durch);
  put_string_usart (outpbuf);

  // Alten Bildschirm wieder zusammensetzen
  Screen ();

  GetNull ();

  // Neue Werte setzen
  Nullpunkt += (Durch / 2);

 }

/************************************************************************
 *
 * Main
 *
 ************************************************************************/
void main (void)
{
  char Taste;

  Init ();
  GetNull ();

  while (1)
   {

     Empfang ();                                       // Daten reinholen

     // Ergebniss seriell ausgeben
     // sprintf (outpbuf, " %d\n\r", Messpunkt);
     // put_string_usart (outpbuf);

     Taste = GetTaste ();

     if (Taste != 0)
      {
       sprintf (outpbuf, " # %d ", Taste);
       put_string_usart (outpbuf);
      }

     // Ergebnisse auf LCD anzeigen
     ShowSingle ();

     // Wird der Nullpunkt neu gesetzt ?
     if (Taste == 1)
      {
       GetNull ();
       Durchmesser = FALSE;
       Screen ();
      }

     // Wird ein Durchmesser eingegeben ?
     if (Taste == 2)
      {
       sprintf (outpbuf, "Durchmesser\n\r");
       put_string_usart (outpbuf);
       GetDurchmesser ();
      }

     // Wenn einer angegeben wurde, diesen auch anzeigen
     if (Durchmesser) ShowDurchmesser ();

   } // Von While (1)

}

// Thats all folks !


