/**************************************************************************************************************
 *
 * Datei : LCD.C
 *
 * LCD-Steuerung
 *
 * Autoren : Finger
 *
 * Controller : Microchip 18F452, 20 Mhz
 * Einstellungen:  OSC=HS, WDT=OFF, PWT=ON, CP=OFF, BOD=ON, LVP=OFF
 *
 *************************************************************************************************************/

#include	<pic18.h>
#include	"lcd.h"
#include	"delay.h"

// Konstanten und Bit-Deklarationen
static bit LCD_RS	@ ((unsigned)&PORTD*8+6);				// LCD: Register select
static bit LCD_EN	@ ((unsigned)&PORTD*8+7);				// Enable

#define	LCD_STROBE	((LCD_EN = 1),(LCD_EN = 1),(LCD_EN=0))			// LCD-Zweizeilen-Display

/***************************************************************************************************************
 *
 * Verzgern im Millisekundebereich
 *
 ***************************************************************************************************************/
void DelayMilli (unsigned char cnt)
 {
   unsigned char	i;
  	do {
      		i = 4;
      		do {
      			   DelayUs (1000);
      		   } while(--i);
       } while(--cnt);
 }

/**************************************************************************************************************
 *
 * Ein Byte zum LCD schreiben im 4-Bit-Modus.
 *
 **************************************************************************************************************/
void lcd_write (unsigned char c)
 {
	PORTD = (PORTD & 0xF0) |  (c >> 4);
	DelayUs (100);
	LCD_STROBE;
	DelayUs (100);
	PORTD = (PORTD & 0xF0) |  (c & 0x0F);
	DelayUs (100);
	LCD_STROBE;
	DelayUs (100);
 }

/**************************************************************************************************************
 *
 * Cursor ein/ausschalten
 *
 **************************************************************************************************************/
void lcd_cursor_on (char type)
 {
 	LCD_RS = 0;		                         // write control bytes
 	if (type) lcd_write (0x0F);	else       // display on, blink curson on
   lcd_write (0x0C);	                   // display on, blink curson off
	DelayMilli (20);
 }

/**************************************************************************************************************
 *
 * LCD lschen und Cursor auf Null setzen
 *
 **************************************************************************************************************/
void lcd_clear (void)
 {
	LCD_RS = 0;
	lcd_write (0x1);
	DelayMilli (10);
 }


/**************************************************************************************************************
 *
 * String auf LCD ausgeben
 *
 **************************************************************************************************************/
void lcd_puts (const char * s)
 {
	LCD_RS = 1;	                                 // write characters
	while(*s) lcd_write(*s++);
 }

/**************************************************************************************************************
 *
 * Ein Zeichen auf dem Display ausgeben
 *
 **************************************************************************************************************/
void lcd_putch (char c)
 {
	LCD_RS = 1;	                                 // write characters
	PORTD = (PORTD & 0xF0) |  (c >> 4);
	DelayUs(50);
	LCD_STROBE;
	DelayUs(50);
	PORTD = (PORTD & 0xF0) |  (c & 0x0F);
	DelayUs (50);
	LCD_STROBE;
	DelayUs(50);
 }

/**************************************************************************************************************
 *
 * Cursor positionieren
 *
 **************************************************************************************************************/
void lcd_goto (unsigned char pos)
 {
	LCD_RS = 0;
	lcd_write (0x80+pos);
 }

/***************************
 *
 * Positioniert den Cursor
 * (0,0) = Links oben
 *
 ***************************/
void lcd_cursorpos (char zeile, char spalte)
 {
  switch (zeile)
   {
    case 0 : { lcd_goto (spalte); break; }
    case 1 : { lcd_goto (0x40 + spalte); break; }
    case 2 : { lcd_goto (0x14 + spalte); break; }
    case 3 : { lcd_goto (0x54 + spalte); break; }
   }
 }

/**************************************************************************************************************
 *
 * LCD initialisieren
 *
 **************************************************************************************************************/
void lcd_init (void)
 {
	LCD_RS = 0;		    // write control bytes
	DelayMilli (20);		  // power on delay
	PORTD = 0x3;		  // attention!
	LCD_STROBE;
	DelayMilli (10);
	LCD_STROBE;
	DelayUs (100);
	LCD_STROBE;
	DelayMilli (10);
	PORTD = 0x2;		  // set 4 bit mode
	LCD_STROBE;
	DelayUs (80);
	lcd_write (0x28);	// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write (0x08);	// display off
	lcd_write (0x0C);	// display on, blink curson off
	lcd_write (0x06);	// entry mode
	DelayMilli (20);

 }

