/**************************************************************************************************************
 *
 * Datei : serial.c
 *
 * Autor : Finger
 *
 * Controller : Microchip 18F452, 20 Mhz
 * Einstellungen:  OSC=HS, WDT=OFF, PWT=ON, CP=OFF, BOD=ON, LVP=OFF
 *
 *************************************************************************************************************/

#include "serial.h"
#include "defines.h"
#include <pic18.h>
#include <string.h>

#define Empfangspuffergroesse 20

char usart_empfangspuffer [Empfangspuffergroesse];          // Puffer fr empfangene Zeichen
char usart_schreibzeiger = 0;                               // Zeiger auf den Empfangspuffer (schreiben)
char usart_lesezeiger = 0;                                  // Zeiger auf den Empfangspuffer (lesen)

char cdummy;                                                // Empfangsregister auslesen
char zeiger = 0;                                            // Lasst uns in die Zukunft schauen....

/*****************************************************************************************************************
 *
 * USART-ISR, hngt im Main in der zentralen ISR
 *
 *****************************************************************************************************************/
void isr_usart (void)
 {
   //----------------------------
   // Receive complete-Interrupt
   //----------------------------
   if (RCIF)
    {
     // Nchste Schreibposition ermitteln
     zeiger = usart_schreibzeiger;
     zeiger++;
     if (zeiger == Empfangspuffergroesse) zeiger = 0;

     // Ist das Pufferende erreicht ? Dann Zeichen verwerfen
     if (zeiger == usart_lesezeiger) cdummy = RCREG; else
      {
       usart_empfangspuffer [usart_schreibzeiger] = RCREG;    // Zeichen speichern
       usart_schreibzeiger = zeiger;                          // Neuen Zeiger setzen
      }

    } // From if (RCIF)

 }

/*****************************************************************************************************************
 *
 * String nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void put_string_usart (const char *data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (FERR) cdummy = RCREG;
  if (OERR)
  	{
    	CREN=0;
    	CREN=1;
  	}
  	do
  	 {
  		while (!TRMT);
      TXREG = *data;
  	 } while ( *++data );
}

/*****************************************************************************************************************
 *
 * Char nach USART schreiben, nicht ISR-gesteuert
 *
 *****************************************************************************************************************/
void put_char_usart (char data)
{
  char cdummy;                                   // Dummyaufnahme des Empfangsregister

  if (FERR) cdummy = RCREG;
  if (OERR)
  	{
    	CREN=0;
    	CREN=1;
  	}
   while (!TRMT);
   TXREG = data;
}

/*****************************************************************************************************************
 *
 * Zeichen aus USART abholen, liefert 0, falls kein Zeichen vorhanden
 *
 *****************************************************************************************************************/
char get_char_usart (void)
 {
  char dummy = 0;

  // Sind berhaupt Zeichen vorhanden ?
  if (usart_lesezeiger != usart_schreibzeiger)
   {
    dummy = usart_empfangspuffer [usart_lesezeiger];
    usart_lesezeiger++;
    if (usart_lesezeiger == Empfangspuffergroesse) usart_lesezeiger = 0;
    return (dummy);
   } else return (0);

 }

/*****************************************************************************************************************
 *
 * Puffer leeren
 *
 *****************************************************************************************************************/
void flush_usart (void)
 {
  usart_lesezeiger = usart_schreibzeiger;
 }


/*****************************************************************************************************************
 *
 * UART initialisieren
 *
 *****************************************************************************************************************/
void UART_Init (void)
 {
  // UART-Einstellungen
  SPBRG=32;       								                // USART: 38400 Baud bei 20MHZ - 8Bit-Mode mit BRGH=1
  TXSTA=0x24;									                    // mit BRGH=1
  RCSTA=0x90;
  RCIF=0;										                      // Flag=0;
 }

